﻿define([
    // Dojo
    "dojo/_base/declare",
    "dojo/store/Memory",
    "dojo/store/JsonRest",
    "dijit/registry",
    // Dijit
    "dijit/_TemplatedMixin",
    "dijit/_Container",
    "dijit/layout/_LayoutWidget",
    "dijit/form/ComboBox",
    //"dijit/layout/_ContentPane",
    "dijit/_WidgetsInTemplateMixin",
    "epi-cms/_ContentContextMixin",
    "epi/dependency",
    "epi/routes",
    "epi/shell/command/_WidgetCommandProviderMixin",
    "dojo/text!./AssetErrorLogs.html"
], function (
    // Dojo
    declare,
    Memory,
    JsonRest,
    registry,
    // Dijit
    _TemplatedMixin,
    _Container,
    _LayoutWidget,
    ComboBox,
    _WidgetsInTemplateMixin,
    _ContentContextMixin,
    dependency,
    routes,
    _WidgetCommandProviderMixin,
    template
) {

        return declare(
            [_Container, _LayoutWidget, _TemplatedMixin, _WidgetsInTemplateMixin, _ContentContextMixin, _WidgetCommandProviderMixin], {
                templateString: template,
                widgetsInTemplate: true,
                store: null,
                context: null,

                contextChanged: function (context, callerData) {
                    this.context = context;
                    this.inherited(arguments);
                },

                _getRestPath: function (name) {
                    return routes.getRestPath({ moduleArea: "lionbridgeconnector", storeName: name });
                },

                _store: function () {
                    var registry = dependency.resolve("epi.storeregistry");
                    if (registry._stores["workspace.workspacestore"] === undefined) {
                        registry.create("workspace.asseterrorlogsstore", this._getRestPath("asseterrorlogsstore"));
                        registry.create("workspace.translationprojectasseterrorlogsliststore", this._getRestPath("translationprojectasseterrorlogsliststore"));
                        registry.create("workspace.selecttranslationprojectasssetliststore", this._getRestPath("selecttranslationprojectasssetliststore"));

                    }

                    this.store = this.store || registry.get("workspace.workspacestore");
                    return this.store;
                },


                startup: function () {
                    var self = this;
                    self.inherited(arguments);
                    this._store();

                    LionBridgeAssetConnectorJs.selectAssetStatusServiceUrl = window.location.origin +
                        this._getRestPath("selecttranslationprojectasssetliststore");
                    LionBridgeAssetConnectorJs.assetErrorLogServiceUrl = window.location.origin +
                        this._getRestPath("translationprojectasseterrorlogsliststore");

                    //Object dojo
                    LionBridgeAssetConnectorJs.dependency = dependency;

                    //Data for Asset status combobox
                    var assetStatusStore = new Memory();
                    var assetTypesStore = new Memory();
                    var assetErrorTypesStore = new Memory();
                    var assetLanguagesStore = new Memory();
                    var assetDeliveryStore = new Memory();

                    var asStore = new JsonRest({
                        target: LionBridgeAssetConnectorJs.selectAssetStatusServiceUrl
                    });

                    asStore.query().then(function (results) {
                        assetStatusStore.add({ name: "All", id: "" });

                        if (results !== null && results.AssetStatues !== null) {
                            var assetStatues = results.AssetStatues;
                            for (var i = 0; i < assetStatues.length; i++) {
                                assetStatusStore.add({ name: assetStatues[i].Text, id: assetStatues[i].Value });
                            }
                        }

                        assetTypesStore.add({ name: "All", id: "" });
                        if (results !== null && results.AssetTypes !== null) {
                            var assetTypes = results.AssetTypes;
                            for (var j = 0; j < assetTypes.length; j++) {
                                assetTypesStore.add({ name: assetTypes[j].Text, id: assetTypes[j].Value });
                            }
                        }

                        assetErrorTypesStore.add({ name: "All", id: "" });
                        assetErrorTypesStore.add({ name: "None", id: "None" });
                        assetErrorTypesStore.add({ name: "Translation Error", id: "Translation Error" });
                        assetErrorTypesStore.add({ name: "Sending Error", id: "Sending Error" });
                        assetErrorTypesStore.add({ name: "Importing Error", id: "Importing Error" });
                        assetErrorTypesStore.add({ name: "Publishing Error", id: "Publishing Error" });
                        assetErrorTypesStore.add({ name: "Other Error", id: "Other Error" });

                        assetLanguagesStore.add({ name: "All", id: "" });
                        if (results !== null && results.Languages !== null) {
                            if (results.Languages)
                                for (var j = 0; j < results.Languages.length; j++) {
                                    assetLanguagesStore.add({ name: results.Languages[j].Text, id: results.Languages[j].Value });
                                }
                        }

                        assetDeliveryStore.add({ name: "All", id: "" });
                        assetDeliveryStore.add({ name: "Yes", id: "Yes" });
                        assetDeliveryStore.add({ name: "No", id: "No" });

                        LionBridgeAssetErrorLogsGrid.ComboBox = ComboBox;
                        LionBridgeAssetErrorLogsGrid.assetStatusDataStore = assetStatusStore;
                        LionBridgeAssetErrorLogsGrid.assetTypesDataStore = assetTypesStore;
                        LionBridgeAssetErrorLogsGrid.assetErrorTypesStore = assetErrorTypesStore;
                        LionBridgeAssetErrorLogsGrid.assetLanguagesStore = assetLanguagesStore;
                        LionBridgeAssetErrorLogsGrid.assetDeliveryStore = assetDeliveryStore;
                        LionBridgeAssetErrorLogsGrid.onRenderGrid();
                        LionBridgeAssetErrorLogsGrid.onRegisterClass();

                    });

                },
                postCreate: function () { }
            });
    });

var $n = jQuery.noConflict(true);

var LionBridgeAssetConnectorConstant = LionBridgeAssetConnectorConstant || (function () {

    return {
    };
})();

var LionBridgeAssetConnectorJs = LionBridgeAssetConnectorJs || (function () {

    function onExportCsv() {
        var registry = LionBridgeAssetConnectorJs.dependency.resolve("epi.storeregistry");
        var store = registry.get("workspace.translationprojectasseterrorlogsliststore");

        var queryPrams = LionBridgeAssetErrorLogsGrid.queryPrams;

        store.put(JSON.stringify(queryPrams)).then(function (item) {
            if (item.error) {
                var errorDialog = new LionBridgeAssetConnectorJs.Alert({
                    heading: "Asset logs not exported",
                    description: "Something went wrong when exporting the asset logs due to: " + item.errorMessage
                });
                errorDialog.show();
            } else {

                var binaryString = window.atob(item.result);
                var len = binaryString.length;
                var bytes = new Uint8Array(len);
                for (var i = 0; i < len; i++) {
                    bytes[i] = binaryString.charCodeAt(i);
                }

                var blob = new Blob([bytes.buffer], { type: 'text/plain' });
                if (window.navigator && window.navigator.msSaveOrOpenBlob) {
                    window.navigator.msSaveOrOpenBlob(blob);
                } else {
                    var objectUrl = URL.createObjectURL(blob);
                    var anchor = document.createElement("a");
                    anchor.download = 'AssetErrorlog' + new Date().toLocaleString() + '.csv';
                    anchor.href = objectUrl;
                    anchor.click();
                }
            }
        });
    }

    //Service
    var assetErrorLogServiceUrl;
    var dependency;

    return {
        //Service
        assetErrorLogServiceUrl: assetErrorLogServiceUrl,
        dependency: dependency,
        onExportCsv: onExportCsv
    };
})();


var LionBridgeAssetErrorLogsGrid = LionBridgeAssetErrorLogsGrid || (function () {
    var pageSize = 20;
    var assetErrorLogsTable;
    var projectDataStore;
    var assetStatusDataStore;
    var assetTypesDataStore;
    var assetErrorTypesStore;
    var assetLanguagesStore;
    var assetDeliveryStore;
    var defaultSortColumnNumber = 6;
    var dateColumnNumber = 6;
    var loaded = false;

    var queryPrams = {
        AssetName: null,
        //Type: null,
        ProjectName: null,
        Status: null,
        FromCreatedDate: null,
        ToCreatedDate: null,
        HasError: null,
        IsArchived: false,
        ErrorType: null,
        Language: null,
        Delivery: null
    };

    function onRenderGrid() {

        var AssetColumnSize = [];

        AssetColumnSize["Request ID"] = "250px";
        //AssetColumnSize["Type"] = "60px";
        AssetColumnSize["Project Name"] = "200px";
        AssetColumnSize["Language"] = "60px";
        AssetColumnSize["Error Type"] = "70px";
        AssetColumnSize["Asset Status"] = "70px";
        AssetColumnSize["Delivery"] = "70px";
        AssetColumnSize["Log Details"] = "300px";
        AssetColumnSize["Created"] = "200px";


        //fix duplicate header incase grid exists.
        if ($n('#asseterrorlog702ff2e674297a14add329bda9c7e thead tr').length > 1) {
            $n('#asseterrorlog702ff2e674297a14add329bda9c7e thead tr').each(function (index) {
                if (index > 0) {
                    $n(this).remove();
                }
            });
        }

        $n('#asseterrorlog702ff2e674297a14add329bda9c7e thead tr').clone(true).appendTo('#asseterrorlog702ff2e674297a14add329bda9c7e thead');

        $n('#asseterrorlog702ff2e674297a14add329bda9c7e thead tr:eq(0) th').each(function (i) {
            var title = $n(this).text();

            if (title === 'Asset Status') {
                $n(this).html('<select id="cbAssetStatus" placeholder="' + title + '" style="width:' + AssetColumnSize[title] + ';padding-left: 3px; -webkit-box-sizing: border-box; moz-box-sizing: border-box; box-sizing: border-box;"><option value="">All</option></select>');

                var statusComboBox = new LionBridgeAssetErrorLogsGrid.ComboBox({
                    id: "cbAssetStatus",
                    value: "All",
                    style: { width: "150px" },
                    store: LionBridgeAssetErrorLogsGrid.assetStatusDataStore,
                    searchAttr: "name",
                    onChange: function (status) {
                        queryPrams.Status = status;
                        LionBridgeAssetErrorLogsGrid.assetErrorLogsTable.ajax.reload();
                    }
                }, "cbAssetStatus").startup();

            }
            //else if (title === 'Type') {
            //    $n(this).html('<select id="cbAssetType" placeholder="' + title + '" style="width:' + AssetColumnSize[title] + ';padding-left: 3px; -webkit-box-sizing: border-box; moz-box-sizing: border-box; box-sizing: border-box;"><option value="">All</option></select>');

            //    var assetTypeComboBox = new LionBridgeAssetErrorLogsGrid.ComboBox({
            //        id: "cbAssetType",
            //        value: "All",
            //        style: { width: "80px" },
            //        store: LionBridgeAssetErrorLogsGrid.assetTypesDataStore,
            //        searchAttr: "name",
            //        onChange: function (type) {
            //            queryPrams.Type = type;
            //            LionBridgeAssetErrorLogsGrid.assetErrorLogsTable.ajax.reload();
            //        }
            //    }, "cbAssetType").startup();

            //}
            else if (title === "Created") {
                $n(this).html('<div class="wrapper">' +
                    '<div id ="left">' +
                    '<label for="fromCreatedDate">From </label><input type="date" max="5030-12-31" id="fromCreatedDate" style="width: 148px; padding-left: 3px; border: 1px solid #b4b4b4; -webkit-box-sizing: border-box; moz-box-sizing: border-box; box-sizing: border-box;">' +
                    '</div>' +
                    '<div id="right">' +
                    '<label for="toCreatedDate">To</label><input type="date" max="5030-12-31" id="toCreatedDate" style="width: 148px;padding-left: 3px; border: 1px solid #b4b4b4; -webkit-box-sizing: border-box; moz-box-sizing: border-box; box-sizing: border-box;">' +
                    '</div>' +
                    '</div>');
                $n("#fromCreatedDate").on('change',
                    function () {
                        queryPrams.FromCreatedDate = this.value;
                        LionBridgeAssetErrorLogsGrid.assetErrorLogsTable.ajax.reload();
                        $n("#toCreatedDate").attr("min", this.value);
                    });

                $n("#toCreatedDate").on('change',
                    function () {
                        queryPrams.ToCreatedDate = this.value;
                        $n("#fromCreatedDate").attr("max", this.value);
                        LionBridgeAssetErrorLogsGrid.assetErrorLogsTable.ajax.reload();
                    });
            }
            else if (title === "Request ID") {
                var titleFormat = title;
                $n(this).html('<input type="text" id="assetName" placeholder="' + titleFormat + '" style="width:' + AssetColumnSize[title] + ';padding-left: 3px; height: 26px; -webkit-box-sizing: border-box; moz-box-sizing: border-box; box-sizing: border-box;"/>');
                $n('input', this).on('keyup change',
                    function () {
                        queryPrams.AssetName = this.value;
                        LionBridgeAssetErrorLogsGrid.assetErrorLogsTable.ajax.reload();
                    });
            }
            else if (title === "Job Name") {

                $n(this).html('<input type="text" id="projectName" placeholder="' + title + '" style="width:' + AssetColumnSize[title] + ';padding-left: 3px; height: 26px; -webkit-box-sizing: border-box; moz-box-sizing: border-box; box-sizing: border-box;"/>');
                $n('input', this).on('keyup change',
                    function () {
                        queryPrams.ProjectName = this.value;
                        LionBridgeAssetErrorLogsGrid.assetErrorLogsTable.ajax.reload();
                    });
            }
            else if (title === "Error Type") {
                $n(this).html('<select id="cbAssetErrorType" placeholder="' + title + '" style="width:' + AssetColumnSize[title] + ';padding-left: 3px; -webkit-box-sizing: border-box; moz-box-sizing: border-box; box-sizing: border-box;"><option value="">All</option></select>');

                var assetTypeComboBox = new LionBridgeAssetErrorLogsGrid.ComboBox({
                    id: "cbAssetErrorType",
                    value: "All",
                    style: { width: "80px" },
                    store: LionBridgeAssetErrorLogsGrid.assetErrorTypesStore,
                    searchAttr: "name",
                    onChange: function (type) {
                        queryPrams.ErrorType = type;
                        LionBridgeAssetErrorLogsGrid.assetErrorLogsTable.ajax.reload();
                    }
                }, "cbAssetErrorType").startup();
            }
            else if (title === "Language") {
                $n(this).html('<select id="cbAssetLanguage" placeholder="' + title + '" style="width:' + AssetColumnSize[title] + ';padding-left: 3px; -webkit-box-sizing: border-box; moz-box-sizing: border-box; box-sizing: border-box;"><option value="">All</option></select>');

                var assetTypeComboBox = new LionBridgeAssetErrorLogsGrid.ComboBox({
                    id: "cbAssetLanguage",
                    value: "All",
                    style: { width: "80px" },
                    store: LionBridgeAssetErrorLogsGrid.assetLanguagesStore,
                    searchAttr: "name",
                    onChange: function (type) {
                        queryPrams.Language = type;
                        LionBridgeAssetErrorLogsGrid.assetErrorLogsTable.ajax.reload();
                    }
                }, "cbAssetLanguage").startup();
            }
            else if (title === "Delivery") {
                $n(this).html('<select id="cbAssetDelivery" placeholder="' + title + '" style="width:' + AssetColumnSize[title] + ';padding-left: 3px; -webkit-box-sizing: border-box; moz-box-sizing: border-box; box-sizing: border-box;"><option value="">All</option></select>');

                var assetTypeComboBox = new LionBridgeAssetErrorLogsGrid.ComboBox({
                    id: "cbAssetDelivery",
                    value: "All",
                    style: { width: "80px" },
                    store: LionBridgeAssetErrorLogsGrid.assetDeliveryStore,
                    searchAttr: "name",
                    onChange: function (type) {
                        queryPrams.Delivery = type;
                        LionBridgeAssetErrorLogsGrid.assetErrorLogsTable.ajax.reload();
                    }
                }, "cbAssetDelivery").startup();
            }

            else if (title === "") {
                $n(this).html("");
            }
            else {
                $n(this).html('<input type="hidden" style="width:150px"/>');
            }
        });

        if (loaded) {
            destroy();
        }

        LionBridgeAssetErrorLogsGrid.assetErrorLogsTable = $n('#asseterrorlog702ff2e674297a14add329bda9c7e').DataTable({
            "fixedHeader": true,
            "processing": true,
            "lengthChange": false,
            "fixedColumns": true,
            "pageLength": pageSize,
            "autoWidth": false,
            "serverSide": true,
            "info": false,
            "ajax": {
                "url": LionBridgeAssetConnectorJs.assetErrorLogServiceUrl,
                "data": function (d) {
                    if (d.order.length > 0) {
                        if (d.order[0].column === dateColumnNumber) {
                            d.ordercolumn = d.columns[d.order[0].column].name;
                        } else {
                            d.ordercolumn = d.columns[d.order[0].column].data;
                        }

                        d.orderBy = d.order[0].dir;
                    }

                    //if (queryPrams.Type === "All") {
                    //    queryPrams.Type = null;
                    //}

                    if (queryPrams.Status === "All") {
                        queryPrams.Status = null;
                    }

                    if (queryPrams.ErrorType === "All") {
                        queryPrams.ErrorType = null;
                    }

                    if (queryPrams.Language === "All") {
                        queryPrams.Language = null;
                    }

                    if (queryPrams.Delivery === "All") {
                        queryPrams.Delivery = null;
                    }

                    d.search = JSON.stringify(queryPrams);

                    d.order = '';
                    d.columns = '';

                },
            },
            "columns": [
                { "data": "Name", "className": "assetname", "width": "250px"},
                //{ "data": "Type", "className": "type", "width": "60px" },
                { "data": "ProjectName", "className": "projectname", "width": "200px" },
                { "data": "Language", "className": "language", "width": "60px" },
                { "data": "ErrorType", "className": "errorType", "width": "70px" },
                { "data": "Status", "className": "status", "width": "70px" },
                { "data": "Delivery", "className": "delivery", "width": "70px" },
                {
                    data: {
                        _: "Created.Display",
                        sort: "Created.Timestamp"
                    }
                    , "className": "created"
                    , "name": "CreateDate"
                    , "width": "200px"
                },
                { "data": "ErrorDetails", "className": "errordetails" },
            ],
            "columnDefs": [
                {
                    "targets": [0],
                    "render": asset_cell_assetname_render
                }
            ],
            "order": [[6, "desc"]],
        });

        loaded = true;
        $n('div.dataTables_filter').hide();

        function asset_cell_assetname_render(data, type, row, meta) {
            if (row.Link === undefined || row.Link === '')
                return '';

            return '<a href="' + row.Link + '" rel="nofollow" target="_blank" status="' + row.Status + '" assetId="' + row.AssetID + '">' + row.Name + '</a>';
        }

    }

    function onReloadGrid() {
        if (LionBridgeAssetErrorLogsGrid !== undefined && LionBridgeAssetErrorLogsGrid.assetErrorLogsTable !== undefined) {

            LionBridgeAssetErrorLogsGrid.assetErrorLogsTable.ajax.reload();
            LionBridgeAssetErrorLogsGrid.assetErrorLogsTable.order([defaultSortColumnNumber, 'desc']).draw();
        }
    }

    function destroy() {
        if ($n.fn.DataTable.isDataTable("#asseterrorlog702ff2e674297a14add329bda9c7e")) {
            LionBridgeAssetErrorLogsGrid.assetErrorLogsTable.clear().destroy();
        }
    }

    function onReset() {

        $n('#assetName').val("");
        $n('#projectName').val("");
        $n('#cbAssetStatus').val("All");
        //$n('#cbAssetType').val("All");
        $n('#fromCreatedDate').val("");
        $n('#toCreatedDate').val("");
        $n('#cbAssetErrorType').val("All");
        $n('#cbAssetLanguage').val("All");
        $n('#cbAssetDelivery').val("All");

        queryPrams = {
            AssetName: null,
            Type: null,
            ProjectName: null,
            Status: null,
            FromCreatedDate: null,
            ToCreatedDate: null,
            HasError: null,
            IsArchived: false,
            ErrorType: null,
            Language: null
        };

        if (LionBridgeAssetErrorLogsGrid !== undefined &&
            LionBridgeAssetErrorLogsGrid.assetErrorLogsTable !== undefined) {

            LionBridgeAssetErrorLogsGrid.assetErrorLogsTable.ajax.reload();
            LionBridgeAssetErrorLogsGrid.assetErrorLogsTable.order([defaultSortColumnNumber, 'desc']).draw();
        }
    }

    function refreshChildGrid(item) {
        var table = $n(item).closest('table');
        if (table !== undefined)
            table.DataTable().ajax.reload();
    }

    function onRegisterClass() {
        var form = $n('#asseterrorlog702ff2e674297a14add329bda9c7e').closest('form');
        if (!$n(form).hasClass('lb_asset_error_log'))
            $n(form).addClass('lb_asset_error_log');

        var body = $n('#asseterrorlog702ff2e674297a14add329bda9c7e').closest('body');
        if (!$n(body).hasClass('lb_body_asset_error_log'))
            $n(body).addClass('lb_body_asset_error_log');
    }

    return {
        onRenderGrid: onRenderGrid,
        projectDataStore: projectDataStore,
        assetStatusDataStore: assetStatusDataStore,
        assetTypesDataStore: assetTypesDataStore,
        assetErrorTypesStore: assetErrorTypesStore,
        assetLanguagesStore: assetLanguagesStore,
        assetErrorLogsTable: assetErrorLogsTable,
        assetDeliveryStore: assetDeliveryStore,
        destroy: destroy,
        onReloadGrid: onReloadGrid,
        refreshChildGrid: refreshChildGrid,
        onReset: onReset,
        onRegisterClass: onRegisterClass,
        queryPrams: queryPrams
    };
})();